"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getProjectDirPathInOutDir = exports.loadConfig = exports.mapPaths = void 0;
const file_utils_1 = require("@jfonx/file-utils");
const globby_1 = require("globby");
const path_1 = require("path");
exports.mapPaths = (paths, mapper) => {
    const dest = {};
    Object.keys(paths).forEach(key => {
        dest[key] = paths[key].map(mapper);
    });
    return dest;
};
exports.loadConfig = (file) => {
    const { extends: ext, compilerOptions: { baseUrl, outDir, paths } = {
        baseUrl: undefined,
        outDir: undefined,
        paths: undefined
    } } = file_utils_1.FileUtils.toObject(file);
    const config = {};
    if (baseUrl) {
        config.baseUrl = baseUrl;
    }
    if (outDir) {
        config.outDir = outDir;
    }
    if (paths) {
        config.paths = paths;
    }
    if (ext) {
        const parentConfig = exports.loadConfig(path_1.resolve(path_1.dirname(file), ext));
        return Object.assign(Object.assign({}, parentConfig), config);
    }
    return config;
};
function getProjectDirPathInOutDir(outDir, projectDir) {
    const dirs = globby_1.sync([
        `${outDir}/**/${projectDir}`,
        `!${outDir}/**/${projectDir}/**/${projectDir}`,
        `!${outDir}/**/node_modules`
    ], {
        dot: true,
        onlyDirectories: true
    });
    dirs.sort((dirA, dirB) => {
        return dirB.split('/').length - dirA.split('/').length;
    });
    return dirs[0];
}
exports.getProjectDirPathInOutDir = getProjectDirPathInOutDir;
//# sourceMappingURL=util.js.map